var gFuncData = null;
var gClassData = null;
var gVariablesData = null;
var gClausesData = null;
var gCommandData = null;
var gTabs = null;
var gTabTemplate = "<li><a href='#{href}'>#{label}</a> <span class='ui-icon ui-icon-close' role='presentation'>Remove Tab</span></li>";

function LoadFile(filepath)
{	
	var retVal = null;
	$.ajax(
	{
		url: filepath,
		dataType: "json",
		async: false,
        success: function(result) 
		{
			if(result.isOk == false) {
				alert(result.message);
			} else {
				retVal = result;
			}
		}		
    });
	return retVal;
}

function InitDoc()
{
	$('#live_search').focus();

	gFuncData = LoadFile('functions.json');
	gClassData = LoadFile('classes.json');
	gVariablesData = LoadFile('variables.json');
	gClausesData = LoadFile('clauses.json');
	gCommandData = LoadFile('commands.json');

	BuildNavBar();

	gTabs = $("#tabs").tabs();

	// close icon: removing the tab on click
	gTabs.delegate( "span.ui-icon-close", "click", function()
	{		
		var panelId = $(this).closest("li").remove().attr( "aria-controls" );
		$( "#" + panelId ).remove();
		gTabs.tabs( "refresh" );
	});

	InitSearch();
}

function InitSearch()
{
	var to = false;
	$('#live_search').keyup(function (event)
	{
		if(event.keyCode == 13 || event.keyCode == 114) {
			select_next_search_result();
			return;
		}

		if(to) { clearTimeout(to); }		
		to = setTimeout(function () {
			reset_search_result();
			var v = $('#live_search').val();
			if(gClassData != null) $('#classTree').jstree(true).search(v);
			if(gFuncData != null) $('#functionTree').jstree(true).search(v);
			if(gVariablesData != null) $('#variablesTree').jstree(true).search(v);
			if(gClausesData != null) $('#clausesTree').jstree(true).search(v);
			if(gCommandData != null) $('#commandTree').jstree(true).search(v);
		}, 250);
	});

	$('#live_search').val('');
}

function deselect_all()
{
	if(gClassData != null) $('#classTree').jstree().deselect_all();
	if(gFuncData != null) $('#functionTree').jstree().deselect_all();
	if(gVariablesData != null) $('#variablesTree').jstree().deselect_all();
	if(gClausesData != null) $('#clausesTree').jstree().deselect_all();
	if(gCommandData != null) $('#commandTree').jstree().deselect_all();
}

var active_result = -1;
var search_results = [];

function reset_search_result()
{
	search_results = [];
	active_result = -1;
}

function select_next_search_result()
{
	deselect_all();

	++active_result;
	if(active_result >= search_results.length) {
		active_result = 0;
	}
	$('#'+search_results[active_result]+'_anchor').trigger("click");
	$('#live_search').focus();	
}

function search_complete(nodes, str, res) {
	search_results = search_results.concat(str.res);
}

function BuildFuncOrCommandTree(objName, data)
{
	var ul = '';

	$.each(data, function(group, funcs)
	{
		if(funcs)
		{
			if(group != 'nogroup')
			{
				ul += '<li>' + atob(group);
				ul += '<ul>';
			}

			$.each(funcs, function(func_name, func_obj) 
			{
				ul += '<li><a href="#" onclick="DisplayFunction(' + objName + '.' + group + '.' + func_name + ');">' + func_name + '</a></li>';
			});

			if(group != 'nogroup')
			{
				ul += '</ul></li>';
			}
		}		
	});

	return ul;
}

function BuildNavBar()
{
	if(gClassData != null)
	{
		$('#navbar').append('<h2>Classes</h2>');
		var ul = '<div id="classTree"><ul>';				
		$.each(gClassData, function(key, value) 
		{
			var rootClass = 'gClassData.' + key;

			ul += '<li><a href="#" onclick="DisplayClass(' + rootClass + ');">' + key + '</a>';
			
			ul += '<ul>';
			
			if(value.functions)
			{
				$.each(value.functions, function(group, funcs) 
				{
					if(funcs)
					{
						$.each(funcs, function(func_name, func_obj) 
						{
							ul += '<li><a href="#" onclick="DisplayClass(' + rootClass + ', \'#m_' + clearString(func_name) + '\');">' + func_name + '</a></li>';
						});
					}
				});
			}

			$.each(value.properties, function(key, value) {
				ul += '<li><a href="#" onclick="DisplayClass(' + rootClass + ', \'#p_' + clearString(key) + '\');">' + key + '</a></li>';
			});
			ul += '</ul>';
			ul += '</li>';

		});
		ul += '</ul></div>';
		$('#navbar').append(ul);

		$("#classTree").jstree({ "plugins" : ["themes", "html_data", "ui", "crrm", "search"] });
		$("#classTree").on('search.jstree', search_complete);
	}

	if(gFuncData != null)
	{
		$('#navbar').append('<h2>Functions</h2>');
		var ul = '<div id="functionTree"><ul>';
		ul += BuildFuncOrCommandTree('gFuncData', gFuncData);		
		ul += '</ul></div>';
		$('#navbar').append(ul);

		$("#functionTree").jstree({"plugins" : ["themes", "html_data", "ui", "search"]});
		$("#functionTree").on('search.jstree', search_complete);
	}

	if(gCommandData != null)
	{		
		$('#navbar').append('<h2>Commands</h2>');
		var ul = '<div id="commandTree"><ul>';
		ul += BuildFuncOrCommandTree('gCommandData', gCommandData);		
		ul += '</ul></div>';
		$('#navbar').append(ul);

		$("#commandTree").jstree({"plugins" : ["themes", "html_data", "ui", "search"]});
		$("#commandTree").on('search.jstree', search_complete);
	}

	if(gVariablesData != null)
	{
		$('#navbar').append('<h2>Variables</h2>');
		var ul = '<div id="variablesTree"><ul>';				
		$.each(gVariablesData, function(key, value) {
			ul += '<li><a href="#" onclick="DisplayVariable(gVariablesData.' + key + ');">' + key + '</a></li>';
		});
		ul += '</ul></div>';
		$('#navbar').append(ul);

		$("#variablesTree").jstree({"plugins" : ["themes", "html_data", "ui", "search"]});
		$("#variablesTree").on('search.jstree', search_complete);
	}

	if(gClausesData != null)
	{
		$('#navbar').append('<h2>Clauses</h2>');
		var ul = '<div id="clausesTree"><ul>';				
		$.each(gClausesData, function(key, value) {
			ul += '<li><a href="#" onclick="DisplayClauses(gClausesData.' + key + ');">' + key + '</a></li>';
		});
		ul += '</ul></div>';
		$('#navbar').append(ul);

		$("#clausesTree").jstree({"plugins" : ["themes", "html_data", "ui", "search"]});
		$("#clausesTree").on('search.jstree', search_complete);
	}
}

function clearString(str)
{
	return str ? str.replace(/\W/g, '') : str;
}

function DisplayFunction(funcData)
{	
	if(JumpToTab(funcData.name)) return;

	var html = '';

	html += '<div style="padding-left:5px;">';
	html += '</div>';
	html += '<br>';
	html += '<div class="methode_div methode_div_outer">';
	html += '	<div class="methode_div methode_div_inner">';
	html += '		<b>Description</b>';
	html += '		<pre style="white-space:normal;padding:2px;border:1px solid #5b5b5b;" class="sh_php">';
	
	html += funcData.return.type + ' <strong>' + funcData.name + '</strong>(';	
	
	var i = 0;
	$.each(funcData.parameters, function(key, value) 
	{
		if(i > 0) 
			html += ', ';

		html += value.type + ' ' + value.name;
		i++;
	});
	html += ')';
	

	html += '</pre>';
	html += funcData.description;
	html += '</div>';

	if(funcData.version)
	{
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Version</b><br>';
		html += '		Script version: ' + funcData.version;
		html += '	</div>';
	}

	if(funcData.parameters.length > 0)
	{	
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Parameters</b>';
		html += '		<ul>';

		$.each(funcData.parameters, function(key, value)
		{
			html += '<li><b>' + value.name + '</b> - ' + value.description + '</li>';	
		});
		
		html += '		</ul>';
		html += '	</div>';	
	}

	if(funcData.return.description)
	{
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Return Value</b><br>';
		html += '		<ul>';
		html += '			<li><b>' + funcData.return.type + '</b> - ' + funcData.return.description + '</li>';
		html += '		</ul>';
		html += '	</div>';
	}

	if(funcData.example)
	{	
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Usage</b>';
		html += '		<div class="code_div">' + funcData.example + '</div>';
		html += '	</div>';
	}
	html += '</div>';

	AddTab(funcData.name, html);
	sh_highlightDocument();
}

function DisplayVariable(variableData)
{	
	if(JumpToTab(variableData.name)) return;

	var html = '';

	html += '<div style="padding-left:5px;">';
	html += '</div>';
	html += '<br>';
	html += '<div class="methode_div methode_div_outer">';
	html += '	<div class="methode_div methode_div_inner">';
	html += '		<b>Description</b>';
	html += '		<pre style="white-space:normal;padding:2px;border:1px solid #5b5b5b;" class="sh_php">';
		
	var i = 0;
	$.each(variableData.indices, function(key, value) 
	{
		if(i > 0) 
			html += ', ';

		html += value.type + ' ' + variableData.name + '[\'' + value.name + '\']';
		i++;
	});
	

	html += '</pre>';
	html += variableData.description;
	html += '</div>';

	if(variableData.indices.length > 0)
	{	
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Indices</b>';
		html += '		<ul>';

		$.each(variableData.indices, function(key, value)
		{
			html += '<li><b>' + value.name + '</b> - ' + value.description + '</li>';	
		});
		
		html += '		</ul>';
		html += '	</div>';	
	}

	if(variableData.return.description)
	{
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Return Value</b><br>';
		html += '		<ul>';
		html += '			<li><b>' + variableData.return.type + '</b> - ' + variableData.return.description + '</li>';
		html += '		</ul>';
		html += '	</div>';
	}

	if(variableData.example)
	{	
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Usage</b>';
		html += '		<div class="code_div">' + variableData.example + '</div>';
		html += '	</div>';
	}
	html += '</div>';

	AddTab(variableData.name, html);
	sh_highlightDocument();
}

function DisplayClauses(clauseData)
{	
	if(JumpToTab(clauseData.name)) return;

	var html = '';

	html += '<div style="padding-left:5px;">';
	html += '</div>';
	html += '<br>';
	html += '<div class="methode_div methode_div_outer">';
	html += '	<div class="methode_div methode_div_inner">';
	html += '		<b>Description</b>';
	html += '		<pre style="white-space:normal;padding:2px;border:1px solid #5b5b5b;" class="sh_php">';	
	html += clauseData.return.type + ' <strong>' + clauseData.name + '</strong>';
	html += '		</pre>';
	html += clauseData.description;
	html += '	</div>';

	if(clauseData.return.description)
	{
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Return Value</b><br>';
		html += '		<ul>';
		html += '			<li><b>' + clauseData.return.type + '</b> - ' + clauseData.return.description + '</li>';
		html += '		</ul>';
		html += '	</div>';
	}

	if(clauseData.example)
	{	
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<b>Usage</b>';
		html += '		<div class="code_div">' + clauseData.example + '</div>';
		html += '	</div>';
	}
	html += '</div>';

	AddTab(clauseData.name, html);
	sh_highlightDocument();
}

function DisplayClass(classArr, jumpTo)
{
	var tabName = classArr.name;

	if(JumpToTab(classArr.name))
	{
		if(jumpTo != null) {
			$('#tabs-'+tabName).scrollTo($(jumpTo), 500);
		} else {
			$('#tabs-'+tabName).scrollTo(0, 500);
		}
		return;
	}

	var html = '';
	html += '<div style="padding-left:5px;">';
	html += '<h2>Description</h2>';
	html += classArr.description;
	html += '<br>';	
	
	if(classArr.functions)
	{
		if(typeof classArr.functions.length == 'undefined')
		{
			html += '<h2>Methods</h2>';
			html += '<ul>';
			
			$.each(classArr.functions, function(group_name, func_list)
			{
				$.each(func_list, function(func_name, func)
				{
					html += '<li><a href="#" onclick="$(\'#tabs-'+tabName+'\').scrollTo(\'#m_' + clearString(func_name) + '\', 500);">' + func_name + '</a> - ' + func.s_description + '</li>';
				});
			});
			
			html += '</ul>';
		}
	}
	
	if(classArr.properties)
	{
		if(typeof classArr.properties.length == 'undefined')
		{
			html += '	<h2>Properties</h2>';
			html += '	<ul>';
			$.each(classArr.properties, function(key, value)
			{
				html += '<li><a href="#" onclick="$(\'#tabs-'+tabName+'\').scrollTo(\'#p_' + clearString(key) + '\', 500);">' + key + '</a> - ' + value.s_description + '</li>';
			});
			html += '	</ul>';	
		}
	}

	html += '</div>';
	html += '<br>';

	if(classArr.example)
	{
		html += '<div class="methode_div methode_div_outer">';
		html += '	<h2>Possibilities to use this class</h2>';
		html += '	<div class="methode_div methode_div_inner">';
		html += '		<div class="code_div">' + classArr.example + '</div>';
		html += '	</div>';
		html += '</div>';
	}

	html += '<br>';
	html += '<br>';

	if(classArr.functions)
	{
		$.each(classArr.functions, function(group_name, func_list)
		{		
			$.each(func_list, function(func_name, func)
			{		
				html += '<div class="methode_div methode_div_outer">';
				html += '	<span name="m_' + func.name + '" id="m_' + clearString(func.name) + '">Method <b>' + func.name + '</b></span>';
				html += '	<div class="methode_div methode_div_inner">';
				html += '		<b>Description</b>';
				html += '		<pre style="white-space:normal;padding:2px;border:1px solid #5b5b5b;" class="sh_php">';
				html += '		' + func.return.type + ' <strong>' +classArr.name + '::' + func.name + '</strong>(';
				
				var i = 0;
				$.each(func.parameters, function(key, value) 
				{
					if(i > 0) 
						html += ', ';

					html += value.type + ' ' + value.name;
					i++;
				});
				html += ')';

				html += '</pre>';
				html += func.description;
				html += '</div>';

				if(func.parameters.length > 0)
				{	
					html += '	<div class="methode_div methode_div_inner">';
					html += '		<b>Parameters</b>';
					html += '		<ul>';

					$.each(func.parameters, function(key, value)
					{
						html += '<li><b>' + value.name + '</b> - ' + value.description + '</li>';	
					});
					
					html += '		</ul>';
					html += '	</div>';	
				}

				if(func.return.description)
				{
					html += '	<div class="methode_div methode_div_inner">';
					html += '		<b>Return Value</b><br>';
					html += '		<ul>';
					html += '			<li><b>' + func.return.type + '</b> - ' + func.return.description + '</li>';
					html += '		</ul>';
					html += '	</div>';
				}

				if(func.example)
				{	
					html += '	<div class="methode_div methode_div_inner">';
					html += '		<b>Usage</b>';
					html += '		<div class="code_div">' + func.example + '</div>';
					html += '	</div>';
				}

				html += '</div>';
			});
		});
	}

	html += '<br>';
	html += '<br>';
	
	if(classArr.properties)
	{
		$.each(classArr.properties, function(name, prop)
		{
			html += '<div class="methode_div methode_div_outer">';
			html += '	<span name="p_' + prop.name + '" id="p_' + clearString(prop.name) + '">Property <b>' + prop.name + '</b></span>';
			html += '	<div class="methode_div methode_div_inner">';
			html += '		<b>Description</b>';
			html += '		<pre style="white-space:normal;padding:2px;border:1px solid #5b5b5b;" class="sh_php">';
			html += '		' + prop.type + ' <strong>' + classArr.name + '::' + prop.name + '</strong>';
			html += '		</pre>';
			html += '		' + prop.description;
			html += '	</div>';
			if(prop.example)
			{
				html += '<div class="methode_div methode_div_inner">';
				html += '	<b>Usage</b>';
				html += '	<div class="code_div">' + prop.example + '</div>';
				html += '</div>';
			}
			html += '</div>';
			html += '<br>';
		});
	}

	AddTab(classArr.name, html);
	sh_highlightDocument();

	if(jumpTo != null) {
		$('#tabs-'+clearString(tabName)).scrollTo($(jumpTo), 500);
	} else {
		$('#tabs-'+clearString(tabName)).scrollTo(0, 500);
	}
}

function JumpToTab(label)
{
	var obj = $('#tabs-' + clearString(label));

	if(obj.length > 0)
	{
		gTabs.tabs("option", "active", obj.index() - 1);
		return true;
	}
	return false;
}

function AddTab(label, content)
{	
	var id = "tabs-" + clearString(label);
	var li = $( gTabTemplate.replace( /#\{href\}/g, "#" + id ).replace( /#\{label\}/g, label ));

	gTabs.find(".ui-tabs-nav").append(li);
	gTabs.append("<div style=\"height:94%;overflow-y:auto;\" id='" + id + "'>" + content + "</div>");
	gTabs.tabs("refresh" );
	gTabs.tabs("option", "active", -1); // -1 = last tab
}